# frozen_string_literal: true

# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Auto-generated by gapic-generator-ruby. DO NOT EDIT!


module Google
  module Cloud
    module Dialogflow
      module V2
        module Conversations
          # Path helper methods for the Conversations API.
          module Paths
            ##
            # Create a fully-qualified Conversation resource string.
            #
            # @overload conversation_path(project:, conversation:)
            #   The resource will be in the following format:
            #
            #   `projects/{project}/conversations/{conversation}`
            #
            #   @param project [String]
            #   @param conversation [String]
            #
            # @overload conversation_path(project:, location:, conversation:)
            #   The resource will be in the following format:
            #
            #   `projects/{project}/locations/{location}/conversations/{conversation}`
            #
            #   @param project [String]
            #   @param location [String]
            #   @param conversation [String]
            #
            # @return [::String]
            def conversation_path **args
              resources = {
                "conversation:project" => (proc do |project:, conversation:|
                  raise ::ArgumentError, "project cannot contain /" if project.to_s.include? "/"

                  "projects/#{project}/conversations/#{conversation}"
                end),
                "conversation:location:project" => (proc do |project:, location:, conversation:|
                  raise ::ArgumentError, "project cannot contain /" if project.to_s.include? "/"
                  raise ::ArgumentError, "location cannot contain /" if location.to_s.include? "/"

                  "projects/#{project}/locations/#{location}/conversations/#{conversation}"
                end)
              }

              resource = resources[args.keys.sort.join(":")]
              raise ::ArgumentError, "no resource found for values #{args.keys}" if resource.nil?
              resource.call(**args)
            end

            ##
            # Create a fully-qualified ConversationProfile resource string.
            #
            # @overload conversation_profile_path(project:, conversation_profile:)
            #   The resource will be in the following format:
            #
            #   `projects/{project}/conversationProfiles/{conversation_profile}`
            #
            #   @param project [String]
            #   @param conversation_profile [String]
            #
            # @overload conversation_profile_path(project:, location:, conversation_profile:)
            #   The resource will be in the following format:
            #
            #   `projects/{project}/locations/{location}/conversationProfiles/{conversation_profile}`
            #
            #   @param project [String]
            #   @param location [String]
            #   @param conversation_profile [String]
            #
            # @return [::String]
            def conversation_profile_path **args
              resources = {
                "conversation_profile:project" => (proc do |project:, conversation_profile:|
                  raise ::ArgumentError, "project cannot contain /" if project.to_s.include? "/"

                  "projects/#{project}/conversationProfiles/#{conversation_profile}"
                end),
                "conversation_profile:location:project" => (proc do |project:, location:, conversation_profile:|
                  raise ::ArgumentError, "project cannot contain /" if project.to_s.include? "/"
                  raise ::ArgumentError, "location cannot contain /" if location.to_s.include? "/"

                  "projects/#{project}/locations/#{location}/conversationProfiles/#{conversation_profile}"
                end)
              }

              resource = resources[args.keys.sort.join(":")]
              raise ::ArgumentError, "no resource found for values #{args.keys}" if resource.nil?
              resource.call(**args)
            end

            ##
            # Create a fully-qualified Location resource string.
            #
            # The resource will be in the following format:
            #
            # `projects/{project}/locations/{location}`
            #
            # @param project [String]
            # @param location [String]
            #
            # @return [::String]
            def location_path project:, location:
              raise ::ArgumentError, "project cannot contain /" if project.to_s.include? "/"

              "projects/#{project}/locations/#{location}"
            end

            ##
            # Create a fully-qualified Project resource string.
            #
            # The resource will be in the following format:
            #
            # `projects/{project}`
            #
            # @param project [String]
            #
            # @return [::String]
            def project_path project:
              "projects/#{project}"
            end

            extend self
          end
        end
      end
    end
  end
end
